#import "BatchViewController.h"
#import "Person.h"
#import "Batch.h"
#import "Storage.h"

@interface BatchViewController ()

@end

@implementation BatchViewController
const int kTableViewContentInset = 90;
const int kMaxNumBatches = 100;

- (void)viewDidLoad {
    [super viewDidLoad];

    self.title = @"Batches";
    [self.navigationController setNavigationBarHidden:NO];
    [self initializeDataSource];
    [self.dataSource.fetchedResultsController performFetch:nil];
}

- (void)viewDidDisappear:(BOOL)animated {
    [self.tableView deselectRowAtIndexPath:[self.tableView indexPathForSelectedRow] animated:YES];
}

- (void)initializeDataSource {
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] initWithEntityName:@"Batch"];
    NSSortDescriptor *sortById = [[NSSortDescriptor alloc] initWithKey:@"idName" ascending:NO];

    [fetchRequest setSortDescriptors:@[sortById]];
    [fetchRequest setFetchBatchSize:kMaxNumBatches];

    self.dataSource = [[BatchViewDataSource alloc] initWithRequest:fetchRequest andContext:[[Storage sharedStorage] managedObjectContext]];
    self.tableView.dataSource = self.dataSource;
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    Batch *batch = [self.dataSource.fetchedResultsController objectAtIndexPath:[self.tableView indexPathForSelectedRow]];
    [segue.destinationViewController setBatch:batch];
}

- (IBAction)resetData:(id)sender {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Reset app data" message:@"This will reset all data stored in the app. You will have to resync." delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"Ok", nil];
    [alert show];
}

#pragma - mark FetchResultsController Delegate

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller {
    [self.tableView beginUpdates];
}

#pragma mark - AlertViewDelegate

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex {
    if (buttonIndex == 1) {
        [self clearAllData];
    }
}

- (void)clearAllData {
    NSManagedObjectContext *objectContext = [[Storage sharedStorage] managedObjectContext];
    NSFetchRequest *fetchBatches = [[NSFetchRequest alloc] initWithEntityName:@"Batch"];
    [fetchBatches setIncludesPropertyValues:NO];

    NSFetchRequest *fetchStudents = [[NSFetchRequest alloc] initWithEntityName:@"Person"];
    [fetchStudents setIncludesPropertyValues:NO];

    NSArray *batches = [objectContext executeFetchRequest:fetchBatches error:nil];
    for (Batch *batch in batches) {
        [objectContext deleteObject:batch];
    }

    NSArray *people = [objectContext executeFetchRequest:fetchStudents error:nil];
    for (Person *person in people) {
        [objectContext deleteObject:person];
    }
    NSError *error;
    [[[Storage sharedStorage] managedObjectContext] save:&error];
    [self.navigationController popToRootViewControllerAnimated:YES];
}

- (IBAction)logout:(id)sender {
    [self clearAllData];
}

@end
