#import "LoginViewController.h"
#import "BlockingViewController.h"
#import <CoreData/CoreData.h>
#import <AFNetworking.h>
#import "Parser.h"
#import "Storage.h"

#define PRE_IOS7 ([[[UIDevice currentDevice] systemVersion] floatValue] < 7.0)
#define IOS_7_OR_LATER ([[[UIDevice currentDevice] systemVersion] floatValue] >= 7.0)

@interface LoginViewController ()

@end

@implementation LoginViewController

- (BlockingViewController *)blockingView
{
    if (!_blockingView) {
        _blockingView = [[BlockingViewController alloc] initWithMessage:@"Loading Hacker School Data.."];
        _blockingView.frame = CGRectMake(0, 0, CGRectGetWidth(self.view.frame), CGRectGetHeight(self.view.frame));
    }
    return _blockingView;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self checkForData];
    self.usernameField.delegate = self;
    self.passwordField.delegate = self;
    self.sitesLoaded = 0;
    self.initialLogoFrame = self.logoImage.frame;
    self.initialPasswordFrame = self.passwordField.frame;
    self.initialUsernameFrame = self.usernameField.frame;
    self.initialLoginButtonFrame = self.loginButton.frame;

    if (PRE_IOS7) {
        self.navigationController.navigationBar.tintColor = [UIColor blackColor];
    }
    self.title = @"Hacker School Faces";
    if (CGRectGetHeight([[UIScreen mainScreen] bounds]) < 568) {
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(toggleKeyboard:) name:UIKeyboardDidChangeFrameNotification object:nil];
    }
}

- (void)viewWillAppear:(BOOL)animated {
    if (IOS_7_OR_LATER) {
        [self.navigationController setNavigationBarHidden:YES animated:NO];
    }
}

- (IBAction)login:(id)sender {
    [self.usernameField resignFirstResponder];
    [self.passwordField resignFirstResponder];

    [self blockUI];
    NSURL *url = [NSURL URLWithString:@"https://www.hackerschool.com"];

    AFHTTPClient *httpClient = [[AFHTTPClient alloc] initWithBaseURL:url];

    NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:self.usernameField.text, @"email", self.passwordField.text, @"password", nil];

    [httpClient postPath:@"/sessions" parameters:params success:^(AFHTTPRequestOperation *operation, id responseObject) {
        if ([[operation.response.URL absoluteString] isEqualToString:@"https://www.hackerschool.com/people"]) {
            Parser *parser = [[Parser alloc] init];
            [parser parseData:responseObject];
            [self unblockUI];
            [self performSegueWithIdentifier:@"loginSegue" sender:nil];
        } else {
            [self unblockUI];
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Login Error" message:@"Your password or email was incorrect." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
            [alert show];
        }

    } failure:^(AFHTTPRequestOperation *operation, NSError *error) {
        [self unblockUI];
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Network Error" message:@"There was an error connecting to hacker school. Check your connection and try again." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
        [alert show];
    }];
}

#pragma - mark TextField Delegate

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}

- (void)blockUI {
    [self.view addSubview:self.blockingView];
    [self.blockingView show];
}

- (void)unblockUI {
    [self.blockingView removeFromSuperview];
}

- (void)checkForData {
    NSManagedObjectContext *objectContext = [[Storage sharedStorage] managedObjectContext];
    NSFetchRequest *dataCheck = [[NSFetchRequest alloc] initWithEntityName:@"Batch"];
    NSArray *batches = [objectContext executeFetchRequest:dataCheck error:nil];
    if ([batches count] > 0) {
        [self performSegueWithIdentifier:@"loginSegue" sender:self];
    }
}

-  (BOOL)shouldAutorotate {
    return NO;
}

- (void)toggleKeyboard:(id)notification {
    if (_usernameField.isEditing || _passwordField.isEditing) {
        [UIView animateWithDuration:0.3 animations:^{
            self.logoImage.alpha = 0;
            self.logoImage.frame = CGRectOffset(self.logoImage.frame, 0, -100);
            self.loginButton.frame = CGRectOffset(self.loginButton.frame, 0, -90);
            self.usernameField.frame = CGRectOffset(self.usernameField.frame, 0, -90);
            self.passwordField.frame = CGRectOffset(self.passwordField.frame, 0, -90);
        }];
    } else {
        [UIView animateWithDuration:0.3 animations:^{
            self.logoImage.alpha = 1.0;
            self.logoImage.frame = self.initialLogoFrame;
            self.loginButton.frame = self.initialLoginButtonFrame;
            self.usernameField.frame = self.initialUsernameFrame;
            self.passwordField.frame = self.initialPasswordFrame;
        }];
    }
}

@end
